@@ -1,7 +1,12 @@
-use strict;
+# The "experimental" below is not actually scary.  The feature went on to be
+# de-experimental-ized with no changes and is now on by default in perl v5.24
+# and later. -- rjbs, 2021-03-14
+use 5.020;
 use warnings;
-package App::Cmd::Setup;
-$App::Cmd::Setup::VERSION = '0.330';
+use experimental qw(postderef postderef_qq);
+
+package App::Cmd::Setup 0.337;
+
 # ABSTRACT: helper for setting up App::Cmd classes
 
 #pod =head1 OVERVIEW
@@ -19,7 +24,7 @@ $App::Cmd::Setup::VERSION = '0.330';
 #pod Instead of writing:
 #pod
 #pod   package MyApp;
-#pod   use base 'App::Cmd';
+#pod   use parent 'App::Cmd';
 #pod
 #pod ...you can write:
 #pod
@@ -33,7 +38,15 @@ $App::Cmd::Setup::VERSION = '0.330';
 #pod   package MyApp;
 #pod   use App::Cmd::Setup -app => { plugins => [ 'Prompt' ] };
 #pod
-#pod Plugins are described in L<App::Cmd::Tutorial> and L<App::Cmd::Plugin>.
+#pod Plugins are described in L<App::Cmd::Tutorial>.
+#pod
+#pod Doing this also allows you to override the default configuration passed to
+#pod L<Getopt::Long>. By default, this configuration includes C<pass_through>,
+#pod which allows subdispatch to work correctly. If you are not using subdispatch,
+#pod and want your command to exit on unknown options, you can say:
+#pod
+#pod   package MyApp;
+#pod   use App::Cmd::Setup -app => { getopt_conf => [] };
 #pod
 #pod = when writing abstract base classes for commands
 #pod
@@ -95,13 +108,15 @@ sub import {
 
 sub _app_base_class { 'App::Cmd' }
 
+my %valid_keys = map {; $_ => 1 } qw(plugins getopt_conf);
+
 sub _make_app_class {
   my ($self, $val, $data) = @_;
   my $into = $data->{into};
 
   $val ||= {};
   Carp::confess "invalid argument to -app setup"
-    if grep { $_ ne 'plugins' } keys %$val;
+    if grep { ! $valid_keys{$_} } keys %$val;
 
   Carp::confess "app setup requested on App::Cmd subclass $into"
     if $into->isa('App::Cmd');
@@ -123,7 +138,7 @@ sub _make_app_class {
   my $want_plugin_base = 'App::Cmd::Plugin';
 
   my @plugins;
-  for my $plugin (@{ $val->{plugins} || [] }) {
+  for my $plugin (@{ $val->{plugins} // [] }) {
     $plugin = String::RewritePrefix->rewrite(
       {
         ''  => 'App::Cmd::Plugin::',
@@ -144,6 +159,16 @@ sub _make_app_class {
     as   => '_plugin_plugins',
   });
 
+  if ($val->{getopt_conf}) {
+    my @getopt_conf = @{ $val->{getopt_conf} };
+
+    Sub::Install::install_sub({
+      code => sub { return [ @getopt_conf ] },
+      into => $into,
+      as   => '_getopt_conf',
+    });
+  }
+
   return 1;
 }
 
@@ -185,7 +210,7 @@ sub _make_plugin_class {
   $val->{groups} = [ default => [ -all ] ] unless $val->{groups};
 
   my @exports;
-  for my $pair (@{ Data::OptList::mkopt($val->{exports}) }) {
+  for my $pair (Data::OptList::mkopt($val->{exports})->@*) {
     push @exports, $pair->[0], ($pair->[1] || \'_faux_curried_method');
   }
 
@@ -214,7 +239,7 @@ App::Cmd::Setup - helper for setting up App::Cmd classes
 
 =head1 VERSION
 
-version 0.330
+version 0.337
 
 =head1 OVERVIEW
 
@@ -231,7 +256,7 @@ This class is useful in three scenarios:
 Instead of writing:
 
   package MyApp;
-  use base 'App::Cmd';
+  use parent 'App::Cmd';
 
 ...you can write:
 
@@ -245,7 +270,15 @@ plugins, as in:
   package MyApp;
   use App::Cmd::Setup -app => { plugins => [ 'Prompt' ] };
 
-Plugins are described in L<App::Cmd::Tutorial> and L<App::Cmd::Plugin>.
+Plugins are described in L<App::Cmd::Tutorial>.
+
+Doing this also allows you to override the default configuration passed to
+L<Getopt::Long>. By default, this configuration includes C<pass_through>,
+which allows subdispatch to work correctly. If you are not using subdispatch,
+and want your command to exit on unknown options, you can say:
+
+  package MyApp;
+  use App::Cmd::Setup -app => { getopt_conf => [] };
 
 =item when writing abstract base classes for commands
 
@@ -279,13 +312,23 @@ L<App::Cmd::Plugin>.
 
 =back
 
+=head1 PERL VERSION
+
+This library should run on perls released even a long time ago.  It should
+work on any version of perl released in the last five years.
+
+Although it may work on older versions of perl, no guarantee is made that the
+minimum required version will not be increased.  The version may be increased
+for any reason, and there is no promise that patches will be accepted to
+lower the minimum required perl.
+
 =head1 AUTHOR
 
-Ricardo Signes <rjbs@cpan.org>
+Ricardo Signes <cpan@semiotic.systems>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Ricardo Signes.
+This software is copyright (c) 2024 by Ricardo Signes.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
